@@ -4,13 +4,10 @@ from itertools import combinations
 from collections import Counter
 from bs4 import BeautifulSoup
 
-# TODO # 1: Find the PERFECT regexes.
-# TODO # 2: iFrames are not supported.
-# TODO # 3: FINAL --- Print the output ready in various forms for multiple platforms and languages.
-
 
 class XPanther:
-    def __init__(self, DOM, xml=False, pre_formatted=False, url_input=False, child_method=True, print_output=True, no_digits=False, show_all=False, speed='normal'):
+    def __init__(self, DOM, xml=False, pre_formatted=False, url_input=False, child_method=True, print_output=True,
+                 no_digits=False, show_all=False, speed='normal'):
         self.__dom = DOM
         self.__keys = []
         self.__formatted_html_lines = []
@@ -32,14 +29,14 @@ class XPanther:
 
     def capture(self, target):
         program_return = []
-        formatted_HTML = self.__format_HTML(self.__dom)
-        element_regex, attr_regex, closing_tag_regex = self.__regex_distributor()
+        formatted_HTML = self.__format_html(self.__dom)
+        element_regex, attr_regex, closing_tag_regex = self.__regex_distributor(formatted_HTML)
 
-        self.__process_HTML(formatted_HTML, element_regex, attr_regex, closing_tag_regex)
+        self.__process_html(formatted_HTML, element_regex, attr_regex, closing_tag_regex)
 
         if type(target) == str and re.match(element_regex, target):
             list_of_occurrences = []
-            formatted_element = self.__format_HTML(target, format_override=True)
+            formatted_element = self.__format_html(target, format_override=True)
 
             for line in formatted_element:
                 if re.match(closing_tag_regex, line.lstrip()):
@@ -47,7 +44,8 @@ class XPanther:
                 else:
                     if self.__xml:
                         if not self.__pre_formatted:
-                            occurrence = [i for i, x in enumerate(self.__formatted_html_lines) if Counter(x[:-1]) == Counter(line.lstrip()[:-2])]
+                            occurrence = [i for i, x in enumerate(self.__formatted_html_lines) if
+                                          Counter(x[:-1]) == Counter(line.lstrip()[:-2])]
                         else:
                             occurrence = [i for i, x in enumerate(self.__formatted_html_lines) if
                                           Counter(x) == Counter(line.lstrip())]
@@ -58,11 +56,13 @@ class XPanther:
             indexes_of_element = []
             if len(list_of_occurrences[0]) > 1:
                 if len(formatted_element) > 1:
-                    successive_pattern = lambda lists: set(lists[0]).intersection(*({x - i for x in l} for i, l in enumerate(lists[1:], start=1)))
+                    successive_pattern = lambda lists: set(lists[0]).intersection(
+                        *({x - i for x in l} for i, l in enumerate(lists[1:], start=1)))
                     successive_pattern = [x for x in successive_pattern(list_of_occurrences)]
                     try:
                         if len(successive_pattern) > 1:
-                            self.__print("\033[95m {}\033[00m" .format(f'Element is identical to {len(successive_pattern) - 1} other element/s || w/o context highest index \u2B07 is preferred'))
+                            self.__print("\033[95m {}\033[00m".format(
+                                f'Element is identical to {len(successive_pattern) - 1} other element/s || w/o context highest index \u2B07 is preferred'))
                             self.__identical = True
                         for pattern in successive_pattern:
                             indexes_of_element.append(list_of_occurrences[0].index(pattern))
@@ -93,7 +93,8 @@ class XPanther:
                         type_of_element = 'xml'
                     self.__print("\033[96m {}\033[00m".format(f'\u2B07 {rarity} Element '), newline=False)
                     self.__print("\033[92m {}\033[00m".format(f'{formatted_element[0][:24]}...>'), newline=False)
-                    self.__print("\033[96m {}\033[00m".format(f'with {type_of_element}-index -> ({row_index + 1} / {len(self.__keys)}) \u2B07'))
+                    self.__print("\033[96m {}\033[00m".format(
+                        f'with {type_of_element}-index -> ({row_index + 1} / {len(self.__keys)}) \u2B07'))
                     if element_xpath := self.__controller(self.__target):
                         program_return.append(element_xpath)
                     else:
@@ -106,9 +107,9 @@ class XPanther:
         elif type(target) == int:
             if target <= len(self.__keys):
                 self.__separate_classes_into_subclasses()
-                self.__target = self.__keys[target-1]
+                self.__target = self.__keys[target - 1]
                 self.__original_target = self.__target
-                self.__print("\033[94m {}\033[00m".format(f'index #{target}/{len(self.__keys)} | '), newline=False)
+                self.__print("\033[94m {}\033[00m".format(f'Index #{target}/{len(self.__keys)} | '), newline=False)
                 if program_return := self.__controller(self.__target):
                     return program_return
                 else:
@@ -123,14 +124,14 @@ class XPanther:
 
     def __controller(self, target_elements):
         if unique_elements := self.__find_unique_elements(target_elements):
-            if xpath := self.__find_XPATH(unique_elements, target_elements):
+            if xpath := self.__find_xpath(unique_elements, target_elements):
                 return xpath
         else:
             cut_down_elements = self.__select_attribute_pool_size(target_elements)
             target_elements = cut_down_elements
             if all_combinations := self.__find_combinations(target_elements):
                 if indicators := self.__find_unique_combinations(all_combinations[0], all_combinations[1]):
-                    if xpath := self.__find_XPATH(indicators, target_elements):
+                    if xpath := self.__find_xpath(indicators, target_elements):
                         return xpath
                 else:
                     if not self.__child_method_ban:
@@ -149,16 +150,16 @@ class XPanther:
             else:
                 return False
 
-    def __format_HTML(self, input, format_override=False):
+    def __format_html(self, input_html, format_override=False):
         parser = 'html.parser'
         if self.__xml:
             parser = 'xml'
-        HTML = input
+        HTML = input_html
         if not format_override:
             if not self.__url_input:
-                if '<' and '>' not in input:
+                if '<' and '>' not in input_html:
                     try:
-                        with open(input, 'r', encoding='utf-8', errors='ignore') as textfile:
+                        with open(input_html, 'r', encoding='utf-8', errors='ignore') as textfile:
                             HTML = textfile.read()
                     except FileNotFoundError:
                         self.__print('File Not Found !')
@@ -177,7 +178,7 @@ class XPanther:
 
         return formatted_HTML
 
-    def __process_HTML(self, formatted_HTML, element_regex, attr_regex, closing_tag_regex):
+    def __process_html(self, formatted_HTML, element_regex, attr_regex, closing_tag_regex):
         index = 0
         for line in formatted_HTML:
             if re.match(closing_tag_regex, line.lstrip()):
@@ -190,10 +191,18 @@ class XPanther:
                     self.__keys.append([keys_of_element, len(line) - len(line.lstrip()), index])
                     index += 1
 
-    @staticmethod
-    def __regex_distributor():
+    def __regex_distributor(self, formatted_HTML):
+        attr_regex = '(?:\w+[-.]*)+(?:=+\"[\w\d\s:;,$@#!\[\]^&?%\'*\\/+(){}.=-]*\")*'
+
+        if self.__pre_formatted:
+            for line in formatted_HTML:
+                if '="' in line:
+                    break
+                elif "='" in line:
+                    attr_regex = "(?:\w+[-.]*)+(?:=+\'[\w\d\s:;,$@#!\[\]^&?%\"*\\/+(){}.=-]*\')*"
+                    break
+
         element_regex = '<.*?>'
-        attr_regex = '(?:\w+[-.]*)+(?:=+[\'\"][\w\d\s:;,$@#!\[\]^&?%*/+(){}.=-]*[\'\"])*'
         closing_tag_regex = '</.*?>'
 
         return element_regex, attr_regex, closing_tag_regex
@@ -271,13 +280,13 @@ class XPanther:
             speed = 35
         elif self.__speed == 'fast':
             speed = 15
-        elif type(self.__speed) == int and self.__speed <= 50:
+        elif type(self.__speed) == int and int(self.__speed) <= 50:
             speed = self.__speed
         else:
             speed = 25
         return [elements[0][:speed], elements[1], elements[2]]
 
-    def __format_XPATH(self, unique_elements, all_elements):
+    def __format_xpath(self, unique_elements, all_elements):
         combination = False
         if self.__identifier_method == 'combination':
             combination = True
@@ -342,8 +351,8 @@ class XPanther:
             self.__print(f'Element\'s --{all_elements[0]}-- format is not normal !')
             return False
 
-    def __find_XPATH(self, unique_elements, target_elements):
-        if result := self.__format_XPATH(unique_elements, target_elements):
+    def __find_xpath(self, unique_elements, target_elements):
+        if result := self.__format_xpath(unique_elements, target_elements):
             self.__gen = []
             self.__children_index = []
             return result
